@@ -163,20 +163,20 @@ pub mod op {
     pub const DIVS_ADDR_NUM_WORD: &str = "DIVS.W";
     pub const DIVS_ADDR_ADDR_BYTE: &str = "DIVS.B";
     pub const DIVS_ADDR_ADDR_WORD: &str = "DIVS.W";
-    pub const JMP_REG: &str = "JMP.B";
-    pub const JMP_ADDR: &str = "JMP.B";
-    pub const JL_REG: &str = "JL.B";
-    pub const JL_ADDR: &str = "JL.B";
-    pub const JLE_REG: &str = "JLE.B";
-    pub const JLE_ADDR: &str = "JLE.B";
-    pub const JG_REG: &str = "JG.B";
-    pub const JG_ADDR: &str = "JG.B";
-    pub const JGE_REG: &str = "JGE.B";
-    pub const JGE_ADDR: &str = "JGE.B";
-    pub const JE_REG: &str = "JE.B";
-    pub const JE_ADDR: &str = "JE.B";
-    pub const JNE_REG: &str = "JNE.B";
-    pub const JNE_ADDR: &str = "JNE.B";
+    pub const JMP_REG: &str = "JMP";
+    pub const JMP_ADDR: &str = "JMP";
+    pub const JL_REG: &str = "JL";
+    pub const JL_ADDR: &str = "JL";
+    pub const JLE_REG: &str = "JLE";
+    pub const JLE_ADDR: &str = "JLE";
+    pub const JG_REG: &str = "JG";
+    pub const JG_ADDR: &str = "JG";
+    pub const JGE_REG: &str = "JGE";
+    pub const JGE_ADDR: &str = "JGE";
+    pub const JE_REG: &str = "JE";
+    pub const JE_ADDR: &str = "JE";
+    pub const JNE_REG: &str = "JNE";
+    pub const JNE_ADDR: &str = "JNE";
     pub const CMP_REG_NUM_BYTE: &str = "CMP.B";
     pub const CMP_REG_NUM_WORD: &str = "CMP.W";
     pub const CMP_REG_REG_BYTE: &str = "CMP.B";
@@ -378,20 +378,20 @@ pub mod full {
     pub const DIVS_ADDR_NUM_WORD: &str = "DIVS.W (A,N)";
     pub const DIVS_ADDR_ADDR_BYTE: &str = "DIVS.B (A,A)";
     pub const DIVS_ADDR_ADDR_WORD: &str = "DIVS.W (A,A)";
-    pub const JMP_REG: &str = "JMP.B (R)";
-    pub const JMP_ADDR: &str = "JMP.B (A)";
-    pub const JL_REG: &str = "JL.B (R)";
-    pub const JL_ADDR: &str = "JL.B (A)";
-    pub const JLE_REG: &str = "JLE.B (R)";
-    pub const JLE_ADDR: &str = "JLE.B (A)";
-    pub const JG_REG: &str = "JG.B (R)";
-    pub const JG_ADDR: &str = "JG.B (A)";
-    pub const JGE_REG: &str = "JGE.B (R)";
-    pub const JGE_ADDR: &str = "JGE.B (A)";
-    pub const JE_REG: &str = "JE.B (R)";
-    pub const JE_ADDR: &str = "JE.B (A)";
-    pub const JNE_REG: &str = "JNE.B (R)";
-    pub const JNE_ADDR: &str = "JNE.B (A)";
+    pub const JMP_REG: &str = "JMP (R)";
+    pub const JMP_ADDR: &str = "JMP (A)";
+    pub const JL_REG: &str = "JL (R)";
+    pub const JL_ADDR: &str = "JL (A)";
+    pub const JLE_REG: &str = "JLE (R)";
+    pub const JLE_ADDR: &str = "JLE (A)";
+    pub const JG_REG: &str = "JG (R)";
+    pub const JG_ADDR: &str = "JG (A)";
+    pub const JGE_REG: &str = "JGE (R)";
+    pub const JGE_ADDR: &str = "JGE (A)";
+    pub const JE_REG: &str = "JE (R)";
+    pub const JE_ADDR: &str = "JE (A)";
+    pub const JNE_REG: &str = "JNE (R)";
+    pub const JNE_ADDR: &str = "JNE (A)";
     pub const CMP_REG_NUM_BYTE: &str = "CMP.B (R,N)";
     pub const CMP_REG_NUM_WORD: &str = "CMP.W (R,N)";
     pub const CMP_REG_REG_BYTE: &str = "CMP.B (R,R)";
