@@ -151,8 +151,8 @@ pub const AND_REG_REG_BYTE: u8 = 0x4A;
 pub const AND_REG_REG_WORD: u8 = 0x4B;
 pub const AND_REG_NUM_BYTE: u8 = 0x4C;
 pub const AND_REG_NUM_WORD: u8 = 0x4D;
-pub const JRF_BYTE: u8 = 0x4E; //jump forward x bytes, seems dangerous - maybe compiler only?
-pub const JRB_BYTE: u8 = 0x4F; //jump backward x bytes, seems dangerous - maybe compiler only?
+pub const JRF_BYTE: u8 = 0x4E;
+pub const JRB_BYTE: u8 = 0x4F;
 
 pub const ASL_REG_NUM_BYTE: u8 = 0x50;
 pub const ASL_REG_NUM_WORD: u8 = 0x51;
